@@ -6,7 +6,7 @@
  */
 
 #ifndef __STDC_FORMAT_MACROS
-#define __STDC_FORMAT_MACROS    
+#define __STDC_FORMAT_MACROS
 #endif
 
 #include <ctype.h>
@@ -171,6 +171,10 @@ cFemonOsd::cFemonOsd()
   m_SvdrpPlugin(NULL),
   m_Number(0),
   m_OldNumber(0),
+  m_Quality(0),
+  m_QualityValid(false),
+  m_Strength(0),
+  m_StrengthValid(false),
   m_SNR(0),
   m_SNRValid(false),
   m_Signal(0),
@@ -241,14 +245,15 @@ cFemonOsd::~cFemonOsd(void)
 void cFemonOsd::DrawStatusWindow(void)
 {
   cMutexLock lock(&m_Mutex);
-  cBitmap *bm = NULL;
-  int offset = 0;
-  int x = OSDWIDTH - OSDROUNDING;
-  int y = 0;
-  eTrackType track = cDevice::PrimaryDevice()->GetCurrentAudioTrack();
   cChannel *channel = Channels.GetByNumber(cDevice::CurrentChannel());
 
   if (m_Osd && channel) {
+     cBitmap *bm = NULL;
+     int offset = 0;
+     int x = OSDWIDTH - OSDROUNDING;
+     int y = 0;
+     eTrackType track = cDevice::PrimaryDevice()->GetCurrentAudioTrack();
+
      OSDDRAWSTATUSTITLEBAR(*cString::sprintf("%d%s %s", m_Number ? m_Number : channel->Number(), m_Number ? "-" : "", channel->ShortName(true)));
      if (m_SvdrpFrontend >= 0) {
         bm = &OSDSYMBOL(SYMBOL_SVDRP);
@@ -295,14 +300,34 @@ void cFemonOsd::DrawStatusWindow(void)
         OSDDRAWSTATUSBM(OSDSPACING);
         }
      if (m_Receiver) {
-        if (IS_OSDRESOLUTION(m_Receiver->VideoVerticalSize(), 1080))
-           bm = &OSDSYMBOL(SYMBOL_FORMAT_1080);
-        else if (IS_OSDRESOLUTION(m_Receiver->VideoVerticalSize(), 720))
-           bm = &OSDSYMBOL(SYMBOL_FORMAT_720);
-        else if (IS_OSDRESOLUTION(m_Receiver->VideoVerticalSize(), 576))
-           bm = &OSDSYMBOL(SYMBOL_FORMAT_576);
-        else if (IS_OSDRESOLUTION(m_Receiver->VideoVerticalSize(), 480))
-           bm = &OSDSYMBOL(SYMBOL_FORMAT_480);
+        if (IS_OSDRESOLUTION(m_Receiver->VideoVerticalSize(), 1080)) {
+           switch (m_Receiver->VideoScan()) {
+              case VIDEO_SCAN_INTERLACED:  bm = &OSDSYMBOL(SYMBOL_FORMAT_1080i); break;
+              case VIDEO_SCAN_PROGRESSIVE: bm = &OSDSYMBOL(SYMBOL_FORMAT_1080p); break;
+              default:                     bm = &OSDSYMBOL(SYMBOL_FORMAT_1080);  break;
+              }
+           }
+        else if (IS_OSDRESOLUTION(m_Receiver->VideoVerticalSize(), 720)) {
+           switch (m_Receiver->VideoScan()) {
+              case VIDEO_SCAN_INTERLACED:  bm = &OSDSYMBOL(SYMBOL_FORMAT_720i); break;
+              case VIDEO_SCAN_PROGRESSIVE: bm = &OSDSYMBOL(SYMBOL_FORMAT_720p); break;
+              default:                     bm = &OSDSYMBOL(SYMBOL_FORMAT_720);  break;
+              }
+           }
+        else if (IS_OSDRESOLUTION(m_Receiver->VideoVerticalSize(), 576)) {
+           switch (m_Receiver->VideoScan()) {
+              case VIDEO_SCAN_INTERLACED:  bm = &OSDSYMBOL(SYMBOL_FORMAT_576i); break;
+              case VIDEO_SCAN_PROGRESSIVE: bm = &OSDSYMBOL(SYMBOL_FORMAT_576p); break;
+              default:                     bm = &OSDSYMBOL(SYMBOL_FORMAT_576);  break;
+              }
+           }
+        else if (IS_OSDRESOLUTION(m_Receiver->VideoVerticalSize(), 480)) {
+           switch (m_Receiver->VideoScan()) {
+              case VIDEO_SCAN_INTERLACED:  bm = &OSDSYMBOL(SYMBOL_FORMAT_480i); break;
+              case VIDEO_SCAN_PROGRESSIVE: bm = &OSDSYMBOL(SYMBOL_FORMAT_480p); break;
+              default:                     bm = &OSDSYMBOL(SYMBOL_FORMAT_480);  break;
+              }
+           }
         else
            bm = NULL;
         OSDDRAWSTATUSBM(OSDSPACING);
@@ -332,11 +357,11 @@ void cFemonOsd::DrawStatusWindow(void)
         OSDDRAWSTATUSBM(OSDSPACING);
         }
      offset += OSDROWHEIGHT;
-     if (m_SignalValid)
-        OSDDRAWSTATUSBAR(m_Signal / 655);
+     if (m_StrengthValid)
+        OSDDRAWSTATUSBAR(m_Strength);
      offset += OSDROWHEIGHT;
-     if (m_SNRValid)
-        OSDDRAWSTATUSBAR(m_SNR / 655);
+     if (m_QualityValid)
+        OSDDRAWSTATUSBAR(m_Quality);
      offset += OSDROWHEIGHT;
      OSDDRAWSTATUSVALUES("STR:", m_SignalValid ? *cString::sprintf("%04x", m_Signal) : "", m_SignalValid ? *cString::sprintf("(%2d%%)", m_Signal / 655) : "",
                          "BER:", m_BERValid ? *cString::sprintf("%08x", m_BER) : "", *cString::sprintf("%s:", tr("Video")),
@@ -364,12 +389,13 @@ void cFemonOsd::DrawStatusWindow(void)
 void cFemonOsd::DrawInfoWindow(void)
 {
   cMutexLock lock(&m_Mutex);
-  int offset = 0;
   cChannel *channel = Channels.GetByNumber(cDevice::CurrentChannel());
-  eTrackType track = cDevice::PrimaryDevice()->GetCurrentAudioTrack();
 
   if (m_Osd && channel) {
+     int offset = 0;
+     eTrackType track = cDevice::PrimaryDevice()->GetCurrentAudioTrack();
      cDvbTransponderParameters dtp(channel->Parameters());
+
      switch (m_DisplayMode) {
        case eFemonModeTransponder:
             OSDDRAWINFOTITLEBAR(tr("Transponder Information"));
@@ -393,7 +419,7 @@ void cFemonOsd::DrawInfoWindow(void)
             offset += OSDROWHEIGHT;
             switch (channel->Source() & cSource::st_Mask) {
               case cSource::stSat:
-                   OSDDRAWINFOLINE(*cString::sprintf("DVB-S%s #%d - %s", (m_FrontendInfo.caps & 0x10000000) ? "2" : "", (m_SvdrpFrontend >= 0) ? m_SvdrpFrontend : cDevice::ActualDevice()->CardIndex(), m_FrontendInfo.name));
+                   OSDDRAWINFOLINE(*cString::sprintf("%s #%d - %s", *getSatelliteSystem(dtp.System()), (m_SvdrpFrontend >= 0) ? m_SvdrpFrontend : cDevice::ActualDevice()->CardIndex(), m_FrontendInfo.name));
                    offset += OSDROWHEIGHT;
                    OSDDRAWINFOLEFT( trVDR("Frequency"),    *getFrequencyMHz(channel->Frequency()));
                    OSDDRAWINFORIGHT(trVDR("Source"),       *cSource::ToString(channel->Source()));
@@ -404,7 +430,8 @@ void cFemonOsd::DrawInfoWindow(void)
                    OSDDRAWINFOLEFT( trVDR("Inversion"),    *getInversion(dtp.Inversion()));
                    OSDDRAWINFORIGHT(trVDR("CoderateH"),    *getCoderate(dtp.CoderateH()));
                    offset += OSDROWHEIGHT;
-                   OSDDRAWINFOLEFT( trVDR("System"),       *getSystem(dtp.System()));
+                   OSDDRAWINFOLEFT( trVDR("System"),       *getSatelliteSystem(dtp.System()));
+                   if (dtp.System())
                    OSDDRAWINFORIGHT(trVDR("RollOff"),      *getRollOff(dtp.RollOff()));
                    break;
 
@@ -422,7 +449,7 @@ void cFemonOsd::DrawInfoWindow(void)
                    break;
 
               case cSource::stTerr:
-                   OSDDRAWINFOLINE(*cString::sprintf("DVB-T #%d - %s", (m_SvdrpFrontend >= 0) ? m_SvdrpFrontend : cDevice::ActualDevice()->CardIndex(), m_FrontendInfo.name));
+                   OSDDRAWINFOLINE(*cString::sprintf("%s #%d - %s", *getTerrestrialSystem(dtp.System()), (m_SvdrpFrontend >= 0) ? m_SvdrpFrontend : cDevice::ActualDevice()->CardIndex(), m_FrontendInfo.name));
                    offset += OSDROWHEIGHT;
                    OSDDRAWINFOLEFT( trVDR("Frequency"),    *getFrequencyMHz(channel->Frequency()));
                    OSDDRAWINFORIGHT(trVDR("Transmission"), *getTransmission(dtp.Transmission()));
@@ -435,6 +462,10 @@ void cFemonOsd::DrawInfoWindow(void)
                    offset += OSDROWHEIGHT;
                    OSDDRAWINFOLEFT( trVDR("Hierarchy"),    *getHierarchy(dtp.Hierarchy()));
                    OSDDRAWINFORIGHT(trVDR("Guard"),        *getGuard(dtp.Guard()));
+                   offset += OSDROWHEIGHT;
+                   OSDDRAWINFOLEFT( trVDR("System"),       *getTerrestrialSystem(dtp.System()));
+                   if (dtp.System())
+                   OSDDRAWINFORIGHT(trVDR("PlpId"),        *cString::sprintf("%d", dtp.PlpId()));
                    break;
 
               default:
@@ -519,6 +550,10 @@ void cFemonOsd::Action(void)
     m_SvdrpVideoBitrate = -1.0;
     m_SvdrpAudioBitrate = -1.0;
     if (m_Frontend != -1) {
+       m_Quality = cDevice::ActualDevice()->SignalQuality();
+       m_QualityValid = (m_Quality >= 0);
+       m_Strength = cDevice::ActualDevice()->SignalStrength();
+       m_StrengthValid = (m_Strength >= 0);
        m_FrontendStatusValid = (ioctl(m_Frontend, FE_READ_STATUS, &m_FrontendStatus) >= 0);
        m_SignalValid = (ioctl(m_Frontend, FE_READ_SIGNAL_STRENGTH, &m_Signal) >= 0);
        m_SNRValid = (ioctl(m_Frontend, FE_READ_SNR, &m_SNR) >= 0);
@@ -531,6 +566,8 @@ void cFemonOsd::Action(void)
        cmd.handle = m_SvdrpConnection.handle;
        m_SvdrpPlugin->Service("SvdrpCommand-v1.0", &cmd);
        if (cmd.responseCode == 900) {
+          m_StrengthValid = false;
+          m_QualityValid = false;
           m_FrontendStatusValid = false;
           m_SignalValid = false;
           m_SNRValid = false;
@@ -540,6 +577,14 @@ void cFemonOsd::Action(void)
               const char *s = line->Text();
 	      if (!strncasecmp(s, "CARD:", 5))
                  m_SvdrpFrontend = (int)strtol(s + 5, NULL, 10);
+              else if (!strncasecmp(s, "STRG:", 5)) {
+                 m_Strength = (int)strtol(s + 5, NULL, 10);
+                 m_StrengthValid = (m_Strength >= 0);
+                 }
+              else if (!strncasecmp(s, "QUAL:", 5)) {
+                 m_Quality = (int)strtol(s + 5, NULL, 10);
+                 m_QualityValid = (m_Quality >= 0);
+                 }
               else if (!strncasecmp(s, "TYPE:", 5))
                  m_FrontendInfo.type = (fe_type_t)strtol(s + 5, NULL, 10);
               else if (!strncasecmp(s, "NAME:", 5)) {
@@ -582,8 +627,8 @@ void cFemonOsd::Show(void)
 {
   debug("%s()\n", __PRETTY_FUNCTION__);
   eTrackType track = cDevice::PrimaryDevice()->GetCurrentAudioTrack();
-  cString dev = cString::sprintf(FRONTEND_DEVICE, cDevice::ActualDevice()->CardIndex(), 0);
-  m_Frontend = open(dev, O_RDONLY | O_NONBLOCK);
+  cDvbDevice *dev = dynamic_cast<cDvbDevice*>(cDevice::ActualDevice());
+  m_Frontend = dev ? open(*cString::sprintf(FRONTEND_DEVICE, dev->Adapter(), dev->Frontend()), O_RDONLY | O_NONBLOCK) : -1;
   if (m_Frontend >= 0) {
      if (ioctl(m_Frontend, FE_GET_INFO, &m_FrontendInfo) < 0) {
         if (!femonConfig.usesvdrp)
@@ -640,8 +685,8 @@ void cFemonOsd::ChannelSwitch(const cDevice * device, int channelNumber)
   if (!device->IsPrimaryDevice() || !channelNumber || cDevice::PrimaryDevice()->CurrentChannel() != channelNumber)
      return;
   close(m_Frontend);
-  cString dev = cString::sprintf(FRONTEND_DEVICE, cDevice::ActualDevice()->CardIndex(), 0);
-  m_Frontend = open(dev, O_RDONLY | O_NONBLOCK);
+  cDvbDevice *dev = dynamic_cast<cDvbDevice*>(cDevice::ActualDevice());
+  m_Frontend = dev ? open(*cString::sprintf(FRONTEND_DEVICE, dev->Adapter(), dev->Frontend()), O_RDONLY | O_NONBLOCK) : -1;
   if (m_Frontend >= 0) {
      if (ioctl(m_Frontend, FE_GET_INFO, &m_FrontendInfo) < 0) {
         if (!femonConfig.usesvdrp)
@@ -811,7 +856,7 @@ eOSState cFemonOsd::ProcessKey(eKeys Key)
 {
   eOSState state = cOsdObject::ProcessKey(Key);
   if (state == osUnknown) {
-     switch (Key) {
+     switch (int(Key)) {
        case k0:
             if ((m_Number == 0) && (m_OldNumber != 0)) {
                m_Number = m_OldNumber;
