@@ -5,6 +5,11 @@
  *
  */
 
+#ifndef __STDC_FORMAT_MACROS
+#define __STDC_FORMAT_MACROS
+#endif
+
+#include <stdint.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <sys/ioctl.h>
@@ -60,7 +65,18 @@ static cString getCA(int value)
   return cString::sprintf("%X", value);
 }
 
-cString getFrontendInfo(int cardIndex)
+static const char *getUserString(int Value, const tDvbParameterMap *Map)
+{
+  const tDvbParameterMap *map = Map;
+  while (map && map->userValue != -1) {
+        if (map->driverValue == Value)
+           return map->userString ? trVDR(map->userString) : "---";
+        map++;
+        }
+  return "---";
+}
+
+cString getFrontendInfo(cDvbDevice *device)
 {
   cString info;
   struct dvb_frontend_info value;
@@ -70,20 +86,29 @@ cString getFrontendInfo(int cardIndex)
   uint32_t ber = 0;
   uint32_t unc = 0;
   cChannel *channel = Channels.GetByNumber(cDevice::CurrentChannel());
-  cString dev = cString::sprintf(FRONTEND_DEVICE, cardIndex, 0);
 
-  int fe = open(dev, O_RDONLY | O_NONBLOCK);
+  if (!device)
+     return info;
+
+  int fe = open(*cString::sprintf(FRONTEND_DEVICE, device->Adapter(), device->Frontend()), O_RDONLY | O_NONBLOCK);
   if (fe < 0)
-     return NULL;
-  CHECK(ioctl(fe, FE_GET_INFO, &value));
-  CHECK(ioctl(fe, FE_READ_STATUS, &status));
-  CHECK(ioctl(fe, FE_READ_SIGNAL_STRENGTH, &signal));
-  CHECK(ioctl(fe, FE_READ_SNR, &snr));
-  CHECK(ioctl(fe, FE_READ_BER, &ber));
-  CHECK(ioctl(fe, FE_READ_UNCORRECTED_BLOCKS, &unc));
-  close(fe);
+     return info;
 
-  info = cString::sprintf("CARD:%d\nTYPE:%d\nNAME:%s\nSTAT:%02X\nSGNL:%04X\nSNRA:%04X\nBERA:%08X\nUNCB:%08X", cardIndex, value.type, value.name, status, signal, snr, ber, unc);
+  info = cString::sprintf("CARD:%d\nSTRG:%d\nQUAL:%d", device->CardIndex(), device->SignalStrength(), device->SignalQuality());
+
+  if (ioctl(fe, FE_GET_INFO, &value) >= 0)
+     info = cString::sprintf("%s\nTYPE:%d\nNAME:%s", *info, value.type, value.name);
+  if (ioctl(fe, FE_READ_STATUS, &status) >= 0)
+     info = cString::sprintf("%s\nSTAT:%02X", *info, status);
+  if (ioctl(fe, FE_READ_SIGNAL_STRENGTH, &signal) >= 0)
+     info = cString::sprintf("%s\nSGNL:%04X", *info, signal);
+  if (ioctl(fe, FE_READ_SNR, &snr) >= 0)
+     info = cString::sprintf("%s\nSNRA:%04X", *info, snr);
+  if (ioctl(fe, FE_READ_BER, &ber) >= 0)
+     info = cString::sprintf("%s\nBERA:%08X", *info, ber);
+  if (ioctl(fe, FE_READ_UNCORRECTED_BLOCKS, &unc) >= 0)
+     info = cString::sprintf("%s\nUNCB:%08X", *info, unc);
+  close(fe);
 
   if (cFemonOsd::Instance())
      info = cString::sprintf("%s\nVIBR:%.0f\nAUBR:%.0f\nDDBR:%.0f", *info, cFemonOsd::Instance()->GetVideoBitrate(), cFemonOsd::Instance()->GetAudioBitrate(), cFemonOsd::Instance()->GetDolbyBitrate());
@@ -94,85 +119,99 @@ cString getFrontendInfo(int cardIndex)
   return info;
 }
 
-cString getFrontendName(int cardIndex)
+cString getFrontendName(cDvbDevice *device)
 {
   struct dvb_frontend_info value;
-  cString dev = cString::sprintf(FRONTEND_DEVICE, cardIndex, 0);
 
-  int fe = open(dev, O_RDONLY | O_NONBLOCK);
+  if (!device)
+     return NULL;
+
+  int fe = open(*cString::sprintf(FRONTEND_DEVICE, device->Adapter(), device->Frontend()), O_RDONLY | O_NONBLOCK);
   if (fe < 0)
      return NULL;
-  CHECK(ioctl(fe, FE_GET_INFO, &value));
+  memset(&value, 0, sizeof(value));
+  ioctl(fe, FE_GET_INFO, &value);
   close(fe);
 
-  return (cString::sprintf("%s on device #%d", value.name, cardIndex));
+  return (cString::sprintf("%s on device #%d", value.name, device->CardIndex()));
 }
 
-cString getFrontendStatus(int cardIndex)
+cString getFrontendStatus(cDvbDevice *device)
 {
   fe_status_t value;
-  cString dev = cString::sprintf(FRONTEND_DEVICE, cardIndex, 0);
 
-  int fe = open(dev, O_RDONLY | O_NONBLOCK);
+  if (!device)
+     return NULL;
+
+  int fe = open(*cString::sprintf(FRONTEND_DEVICE, device->Adapter(), device->Frontend()), O_RDONLY | O_NONBLOCK);
   if (fe < 0)
      return NULL;
-  CHECK(ioctl(fe, FE_READ_STATUS, &value));
+  memset(&value, 0, sizeof(value));
+  ioctl(fe, FE_READ_STATUS, &value);
   close(fe);
 
-  return (cString::sprintf("Status %s:%s:%s:%s:%s on device #%d", (value & FE_HAS_LOCK) ? "LOCKED" : "-", (value & FE_HAS_SIGNAL) ? "SIGNAL" : "-", (value & FE_HAS_CARRIER) ? "CARRIER" : "-", (value & FE_HAS_VITERBI) ? "VITERBI" : "-", (value & FE_HAS_SYNC) ? "SYNC" : "-", cardIndex));
+  return (cString::sprintf("Status %s:%s:%s:%s:%s on device #%d", (value & FE_HAS_LOCK) ? "LOCKED" : "-", (value & FE_HAS_SIGNAL) ? "SIGNAL" : "-", (value & FE_HAS_CARRIER) ? "CARRIER" : "-", (value & FE_HAS_VITERBI) ? "VITERBI" : "-", (value & FE_HAS_SYNC) ? "SYNC" : "-", device->CardIndex()));
 }
 
-uint16_t getSignal(int cardIndex)
+uint16_t getSignal(cDvbDevice *device)
 {
   uint16_t value = 0;
-  cString dev = cString::sprintf(FRONTEND_DEVICE, cardIndex, 0);
 
-  int fe = open(dev, O_RDONLY | O_NONBLOCK);
+  if (!device)
+     return (value);
+
+  int fe = open(*cString::sprintf(FRONTEND_DEVICE, device->Adapter(), device->Frontend()), O_RDONLY | O_NONBLOCK);
   if (fe < 0)
      return (value);
-  CHECK(ioctl(fe, FE_READ_SIGNAL_STRENGTH, &value));
+  ioctl(fe, FE_READ_SIGNAL_STRENGTH, &value);
   close(fe);
 
   return (value);
 }
 
-uint16_t getSNR(int cardIndex)
+uint16_t getSNR(cDvbDevice *device)
 {
   uint16_t value = 0;
-  cString dev = cString::sprintf(FRONTEND_DEVICE, cardIndex, 0);
 
-  int fe = open(dev, O_RDONLY | O_NONBLOCK);
+  if (!device)
+     return (value);
+
+  int fe = open(*cString::sprintf(FRONTEND_DEVICE, device->Adapter(), device->Frontend()), O_RDONLY | O_NONBLOCK);
   if (fe < 0)
      return (value);
-  CHECK(ioctl(fe, FE_READ_SNR, &value));
+  ioctl(fe, FE_READ_SNR, &value);
   close(fe);
 
   return (value);
 }
 
-uint32_t getBER(int cardIndex)
+uint32_t getBER(cDvbDevice *device)
 {
   uint32_t value = 0;
-  cString dev = cString::sprintf(FRONTEND_DEVICE, cardIndex, 0);
 
-  int fe = open(dev, O_RDONLY | O_NONBLOCK);
+  if (!device)
+     return (value);
+
+  int fe = open(*cString::sprintf(FRONTEND_DEVICE, device->Adapter(), device->Frontend()), O_RDONLY | O_NONBLOCK);
   if (fe < 0)
      return (value);
-  CHECK(ioctl(fe, FE_READ_BER, &value));
+  ioctl(fe, FE_READ_BER, &value);
   close(fe);
 
   return (value);
 }
 
-uint32_t getUNC(int cardIndex)
+uint32_t getUNC(cDvbDevice *device)
 {
   uint32_t value = 0;
-  cString dev = cString::sprintf(FRONTEND_DEVICE, cardIndex, 0);
 
-  int fe = open(dev, O_RDONLY | O_NONBLOCK);
+  if (!device)
+     return (value);
+
+  int fe = open(*cString::sprintf(FRONTEND_DEVICE, device->Adapter(), device->Frontend()), O_RDONLY | O_NONBLOCK);
   if (fe < 0)
      return (value);
-  CHECK(ioctl(fe, FE_READ_UNCORRECTED_BLOCKS, &value));
+  ioctl(fe, FE_READ_UNCORRECTED_BLOCKS, &value);
   close(fe);
 
   return (value);
@@ -269,6 +308,7 @@ cString getAudioCodec(int value)
     case AUDIO_CODEC_MPEG2_II:  return cString::sprintf("%s", tr("MPEG-2 Layer II"));
     case AUDIO_CODEC_MPEG2_III: return cString::sprintf("%s", tr("MPEG-2 Layer III"));
     case AUDIO_CODEC_HEAAC:     return cString::sprintf("%s", tr("HE-AAC"));
+    case AUDIO_CODEC_LATM:      return cString::sprintf("%s", tr("LATM"));
     default:                    break;
     }
   return cString::sprintf("---");
@@ -288,95 +328,52 @@ cString getAudioChannelMode(int value)
 
 cString getCoderate(int value)
 {
-  switch (value) {
-    case FEC_NONE: return cString::sprintf("%s", trVDR("none"));
-    case FEC_1_2:  return cString::sprintf("1/2");
-    case FEC_2_3:  return cString::sprintf("2/3");
-    case FEC_3_4:  return cString::sprintf("3/4");
-    case FEC_4_5:  return cString::sprintf("4/5");
-    case FEC_5_6:  return cString::sprintf("5/6");
-    case FEC_6_7:  return cString::sprintf("6/7");
-    case FEC_7_8:  return cString::sprintf("7/8");
-    case FEC_8_9:  return cString::sprintf("8/9");
-    case FEC_AUTO: return cString::sprintf("%s", trVDR("auto"));
-    default:       break;
-    }
-  return cString::sprintf("---");
+  return cString::sprintf("%s", getUserString(value, CoderateValues));
 }
 
 cString getTransmission(int value)
 {
-  switch (value) {
-    case TRANSMISSION_MODE_2K:   return cString::sprintf("2K");
-    case TRANSMISSION_MODE_8K:   return cString::sprintf("8K");
-    case TRANSMISSION_MODE_AUTO: return cString::sprintf("%s", trVDR("auto"));
-    default:                     break;
-    }
-  return cString::sprintf("---");
+  return cString::sprintf("%s", getUserString(value, TransmissionValues));
 }
- 
+
 cString getBandwidth(int value)
 {
-  switch (value) {
-    case BANDWIDTH_8_MHZ: return cString::sprintf("8 %s", tr("MHz"));
-    case BANDWIDTH_7_MHZ: return cString::sprintf("7 %s", tr("MHz"));
-    case BANDWIDTH_6_MHZ: return cString::sprintf("6 %s", tr("MHz"));
-    case BANDWIDTH_AUTO:  return cString::sprintf("%s", trVDR("auto"));
-    default:              break;
-    }
-  return cString::sprintf("---");
+  return cString::sprintf("%s", getUserString(value, BandwidthValues));
 }
 
 cString getInversion(int value)
 {
-  switch (value) {
-    case INVERSION_OFF:  return cString::sprintf("%s", tr("off"));
-    case INVERSION_ON:   return cString::sprintf("%s", tr("on"));
-    case INVERSION_AUTO: return cString::sprintf("%s", trVDR("auto"));
-    default:             break;
-    }
-  return cString::sprintf("---");
+  return cString::sprintf("%s", getUserString(value, InversionValues));
 }
 
 cString getHierarchy(int value)
 {
-  switch (value) {
-    case HIERARCHY_NONE: return cString::sprintf("%s", trVDR("none"));
-    case HIERARCHY_1:    return cString::sprintf("1");
-    case HIERARCHY_2:    return cString::sprintf("2");
-    case HIERARCHY_4:    return cString::sprintf("4");
-    case HIERARCHY_AUTO: return cString::sprintf("%s", trVDR("auto"));
-    default:             break;
-    }
-  return cString::sprintf("---");
+  return cString::sprintf("%s", getUserString(value, HierarchyValues));
 }
 
 cString getGuard(int value)
 {
-  switch (value) {
-    case GUARD_INTERVAL_1_32: return cString::sprintf("1/32");
-    case GUARD_INTERVAL_1_16: return cString::sprintf("1/16");
-    case GUARD_INTERVAL_1_8:  return cString::sprintf("1/8");
-    case GUARD_INTERVAL_1_4:  return cString::sprintf("1/4");
-    case GUARD_INTERVAL_AUTO: return cString::sprintf("%s", trVDR("auto"));
-    default:                  break;
-    }
-  return cString::sprintf("---");
+  return cString::sprintf("%s", getUserString(value, GuardValues));
 }
 
 cString getModulation(int value)
 {
-  switch (value) {
-    case QPSK:     return cString::sprintf("QPSK");
-    case QAM_16:   return cString::sprintf("QAM 16");
-    case QAM_32:   return cString::sprintf("QAM 32");
-    case QAM_64:   return cString::sprintf("QAM 64");
-    case QAM_128:  return cString::sprintf("QAM 128");
-    case QAM_256:  return cString::sprintf("QAM 256");
-    case QAM_AUTO: return cString::sprintf("QAM %s", trVDR("auto"));
-    default:       break;
-    }
-  return cString::sprintf("---");
+  return cString::sprintf("%s", getUserString(value, ModulationValues));
+}
+
+cString getTerrestrialSystem(int value)
+{
+  return cString::sprintf("%s", getUserString(value, SystemValuesTerr));
+}
+
+cString getSatelliteSystem(int value)
+{
+  return cString::sprintf("%s", getUserString(value, SystemValuesSat));
+}
+
+cString getRollOff(int value)
+{
+  return cString::sprintf("%s", getUserString(value, RollOffValues));
 }
 
 cString getResolution(int width, int height, int scan)
@@ -559,7 +556,36 @@ cString getBitrateMbits(double value)
 
 cString getBitrateKbits(double value)
 {
-  if (value > 0) 
+  if (value > 0)
      return cString::sprintf("%.0f %s", value / 1000.0, tr("kbit/s"));
   return cString::sprintf("---");
 }
+
+// --- cFemonBitStream -------------------------------------------------------
+
+uint32_t cFemonBitStream::GetUeGolomb()
+{
+  int n = 0;
+
+  while (!GetBit() && (n < 32))
+    n++;
+
+  return (n ? ((1 << n) - 1) + GetBits(n) : 0);
+}
+
+int32_t cFemonBitStream::GetSeGolomb()
+{
+  uint32_t r = GetUeGolomb() + 1;
+
+  return ((r & 1) ? -(r >> 1) : (r >> 1));
+}
+
+void cFemonBitStream::SkipGolomb()
+{
+  int n = 0;
+
+  while (!GetBit() && (n < 32))
+    n++;
+
+  SkipBits(n);
+}
