@@ -10,27 +10,31 @@
 
 #include <stdint.h>
 #include <vdr/channels.h>
+#include <vdr/dvbdevice.h>
+#include <vdr/remux.h>
 #include <vdr/tools.h>
 
 #ifdef DEBUG
-#define Dprintf(x...) printf(x);
+#define debug(x...) dsyslog("FEMON: " x);
+#define error(x...) esyslog("ERROR: " x);
 #else
-#define Dprintf(x...) ;
+#define debug(x...) ;
+#define error(x...) esyslog("ERROR: " x);
 #endif
 
 #define ELEMENTS(x) (sizeof(x) / sizeof(x[0]))
 
 #define FRONTEND_DEVICE "/dev/dvb/adapter%d/frontend%d"
 
-cString getFrontendInfo(int cardIndex = 0);
-cString getFrontendName(int cardIndex = 0);
-cString getFrontendStatus(int cardIndex = 0);
+cString getFrontendInfo(cDvbDevice *device);
+cString getFrontendName(cDvbDevice *device);
+cString getFrontendStatus(cDvbDevice *device);
 
-uint16_t getSNR(int cardIndex = 0);
-uint16_t getSignal(int cardIndex = 0);
+uint16_t getSNR(cDvbDevice *device);
+uint16_t getSignal(cDvbDevice *device);
 
-uint32_t getBER(int cardIndex = 0);
-uint32_t getUNC(int cardIndex = 0);
+uint32_t getBER(cDvbDevice *device);
+uint32_t getUNC(cDvbDevice *device);
 
 cString getApids(const cChannel *channel);
 cString getDpids(const cChannel *channel);
@@ -48,6 +52,9 @@ cString getInversion(int value);
 cString getHierarchy(int value);
 cString getGuard(int value);
 cString getModulation(int value);
+cString getTerrestrialSystem(int value);
+cString getSatelliteSystem(int value);
+cString getRollOff(int value);
 cString getResolution(int width, int height, int scan);
 cString getAspectRatio(int value);
 cString getVideoFormat(int value);
@@ -66,4 +73,14 @@ cString getVideoBitrate(double value, double stream);
 cString getBitrateMbits(double value);
 cString getBitrateKbits(double value);
 
+class cFemonBitStream : public cBitStream {
+public:
+  cFemonBitStream(const uint8_t *Data, const int Length) : cBitStream(Data, Length) {}
+  uint32_t       GetUeGolomb();
+  int32_t        GetSeGolomb();
+  void           SkipGolomb();
+  void           SkipUeGolomb() { SkipGolomb(); }
+  void           SkipSeGolomb() { SkipGolomb(); }
+  };
+
 #endif // __FEMONTOOLS_H
